﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"

using namespace SoundCharacteristics;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

MainPage::MainPage()
{
	InitializeComponent();

    // Create an IXAudio2 object
    HRESULT hr = XAudio2Create(&pXAudio2);

    if (FAILED(hr))
    {
        errorText->Text = "XAudio2Create failure: " + hr.ToString();
        return;
    }

    // Create a mastering voice
    hr = pXAudio2->CreateMasteringVoice(&pMasteringVoice);

    if (FAILED(hr))
    {
        errorText->Text = "CreateMasteringVoice failure: " + hr.ToString();
        return;
    }

    // Create a source voice
    WAVEFORMATEX waveformat;
    waveformat.wFormatTag = WAVE_FORMAT_PCM;
    waveformat.nChannels = channels;
    waveformat.nSamplesPerSec = sampleRate;
    waveformat.wBitsPerSample = 16;
    waveformat.nBlockAlign = waveformat.nChannels * waveformat.wBitsPerSample / 8;
    waveformat.nAvgBytesPerSec = waveformat.nSamplesPerSec * waveformat.nBlockAlign;
    waveformat.cbSize = 0;
	
    hr = pXAudio2->CreateSourceVoice(&pSourceVoice, &waveformat);

    if (FAILED(hr))
    {
        errorText->Text = "CreateSourceVoice failure: " + hr.ToString();
        return;
    }

    // Start the source voice 
    hr = pSourceVoice->Start();

    if (FAILED(hr))
    {
        errorText->Text = "Start failure: " + hr.ToString();
        return;
    }

    // Create samples for the volume demo
    for (int index = 0, sample = 0; sample < samples; sample++)
    {
        double t = 1;

        if (sample < 2 * samples / 5)
            t = sample / (2.0 * samples / 5);

        else if (sample > 3 * samples / 5)
            t = (samples - sample) / (2.0 * samples / 5);

        double amplitude = pow(2, 15 * t) - 1;
        short waveform = sample % 100 < 50 ? 1 : -1;
        short value = short(amplitude * waveform);

        volumeSoundData[index++] = value;
        volumeSoundData[index++] = value;
    }

    // Create samples for pitch demo
    double angle = 0;

    for (int index = 0, sample = 0; sample < samples; sample++)
    {
        double t = 1;

        if (sample < 2 * samples / 5)
            t = sample / (2.0 * samples / 5);

        else if (sample > 3 * samples / 5)
            t = (samples - sample) / (2.0 * samples / 5);

        double frequency = 220 * pow(2, 2 * t);
        double angleIncrement = 360 * frequency / waveformat.nSamplesPerSec;
        angle += angleIncrement;

        while (angle > 360)
            angle -= 360;
           
        short value = angle < 180 ? 32767 : -32767;
        pitchSoundData[index++] = value;
        pitchSoundData[index++] = value;
    }

    // Create samples for space demo
    for (int index = 0, sample = 0; sample < samples; sample++)
    {
        double t = 0;

        if (sample < samples / 5)
            t = -1.0 * sample / (samples / 5);

        else if (sample < 3 * samples / 5)
            t = -1 + (sample - samples / 5.0) / (samples / 5);

        else if (sample < 4 * samples / 5)
            t = 1 - (sample - 3.0 * samples / 5) / (samples / 5);

        short value = sample % 100 < 50 ? 32767 : -32768;
        short left = short(value * (1 - t) / 2);
        short right = short(value * (1 + t) / 2);

        spaceSoundData[index++] = left;
        spaceSoundData[index++] = right;
    }

    // Create samples for timbre demo
    for (int index = 0, sample = 0; sample < samples; sample++)
    {
        double t = (double)sample / samples;

        double sine = sin(sample % 100 * 2 * 3.14159 / 100);
        double square = sample % 100 < 50 ? 1 : -1;
        double triangle = 2 - sample % 100 / 25.0;

        if (triangle > 1)
            triangle = 2 - triangle;

        else if (triangle < -1)
            triangle = -2 - triangle;

        double sawtooth = 1 - sample % 100 / 50.0;
        double waveform = sawtooth;

        if (t < 0.2)
        {
            waveform = sine;
        }
        else if (t < 0.4)
        {
            double weight = (t - 0.2) / 0.2;
            waveform = weight * triangle + (1 - weight) * sine;
        }
        else if (t < 0.6)
        {
            double weight = (t - 0.4) / 0.2;
            waveform = weight * square + (1 - weight) * triangle;
        }
        else if (t < 0.8)
        {
            double weight = (t - 0.6) / 0.2;
            waveform = weight * sawtooth + (1 - weight) * square;
        }

        short value = short(32767 * waveform);
        timbreSoundData[index++] = value;
        timbreSoundData[index++] = value;
    }
}

void MainPage::OnButtonClick(Object^ sender, RoutedEventArgs^ args)
{
    String^ tag = dynamic_cast<Button^>(sender)->Tag->ToString();
    short * soundData = nullptr;

    if (tag == "volume")
        soundData = volumeSoundData;
    else if (tag == "pitch")
        soundData = pitchSoundData;
    else if (tag == "space")
        soundData = spaceSoundData;
    else if (tag == "timbre")
        soundData = timbreSoundData;

    // Create a buffer to reference the array
    XAUDIO2_BUFFER buffer = { 0 };
    buffer.AudioBytes = sizeof(short) * channels * samples;
    buffer.pAudioData = (byte *) soundData;
    buffer.Flags = XAUDIO2_END_OF_STREAM;
    buffer.PlayBegin = 0;
    buffer.PlayLength = samples;

    // Submit the buffer
    HRESULT hr = pSourceVoice->SubmitSourceBuffer(&buffer);

    if (FAILED(hr))
    {
        errorText->Text = "SubmitSourceBuffer failure: " + hr.ToString();
        return;
    }
}
